/*
  Copyright (C) 2009 Gary Briggs <chunky@icculus.org>

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

  Gary Briggs <chunky@icculus.org>

*/


#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include "v1cs.h"

// http://www.v1connect.com/Protocol.aspx

int main(int argc, char **argv) {
	char *device = strdup(V1CS_DEFAULT_DEVICE);

	int turn_on = 0;
	int turn_off = 0;
	int mute = 0;
	int change_mode = 0;
	int listen = 0;

	int mustexit = 0;

	int optc;
	while ((optc = getopt_long (argc, argv, shortopts, longopts, NULL)) != -1) {
		switch (optc) {
			case 'h':
				printhelp(argv[0]);
				mustexit = 1;
				break;
			case 'v':
				printversion();
				mustexit = 1;
				break;
			case 'd':
				if(NULL != device) {
					free(device);
				}
				device = strdup(optarg);
				break;
			case 'o':
				change_mode = 1;
				break;
			case '0':
				turn_off = 1;
				break;
			case '1':
				turn_on = 1;
				break;
			case 'l':
				listen = 1;
				break;
			case 'm':
				mute = 1;
				break;
		}
	}

	if(turn_on && turn_off) {
		fprintf(stderr, "Turn on and off. Won't do what you want, but will honor\n");
	}

	if(mustexit) exit(1);

	int fd = v1cs_open(device);
	if(-1 == fd) {
		fprintf(stderr, "Can't continue without device. Exiting\n");
		exit(1);
	}

	if(turn_on) {
		v1cs_on(fd);
	}

	if(turn_off) {
		v1cs_off(fd);
	}

	if(change_mode) {
		v1cs_mode(fd);
	}

	if(mute) {
		v1cs_mute(fd);
	}

	if(listen) {
		int loud;
		int strength;
		int ledX;
		int ledK;
		int ledKA;
		int ledLaser;
		int eightsegment[8];
		int frontarrow;
		int sidearrow;
		int reararrow;

		int firsttime = 1;
		while(0 == v1cs_listen(fd,firsttime,
			&loud, &strength,
			&ledX, &ledK, &ledKA, &ledLaser,
			eightsegment,
			&frontarrow, &sidearrow, &reararrow)) {
			
			firsttime = 0;

			char leds[32];
			snprintf(leds, sizeof(leds), "X:%s, K:%s, KA:%s, Laser:%s",
				ledX?"On":"Off", ledK?"On":"Off", ledKA?"On":"Off",
				ledLaser?"On":"Off");

			char arrows[16];
			snprintf(arrows, sizeof(arrows), "Front:%s, Side:%s, Rear:%s",
				frontarrow?"On":"Off", sidearrow?"On":"Off",
				reararrow?"On":"Off");

			printf("Loud: %i\n"
				"Strength: %i\n"
				"Leds: %s\n"
				"EightSegment: %i%i%i%i%i%i%i%i\n"
				"Arrows: %s\n\n",
				loud, strength,
				leds,
				eightsegment[0], eightsegment[1], eightsegment[2],
				eightsegment[3], eightsegment[4], eightsegment[5],
				eightsegment[6], eightsegment[7],
				arrows);
		}
	}

	v1cs_close(fd);

	return 0;
}

int v1cs_open(const char *dev) {
	int fd;

	fd = open(dev, O_RDWR | O_NOCTTY | O_NDELAY);
	if(-1 == fd) {
		perror("Error opening device");
		return -1;
	}

	return fd;
}

void v1cs_close(int fd) {
	close(fd);
}

void v1cs_on(int fd) {
	write(fd, "p", 1);
}

void v1cs_off(int fd) {
	write(fd, "o", 1);
}

void v1cs_mute(int fd) {
	write(fd, "m", 1);
}

void v1cs_mode(int fd) {
	write(fd, "c", 1);
}

int v1cs_listen(int fd, int firstcall,
		int *loud, int *strength,
		int *ledX, int *ledK, int *ledKA, int *ledLaser,
		int eightsegment[8],
		int *frontarrow, int *sidearrow, int *reararrow) {

	char onebyte;
	int retval;

	char buf[17];
	int currbyte = 0;

	// Read up to first space if this is the  first time today
	if(firstcall)
		while(1 == (retval = read(fd, &onebyte, 1)) && ' ' != onebyte);

	if(0 > retval) {
		perror("Error reading from v1 serial port (1)");
		return 1;
	}

	while(1 == (retval = read(fd, buf+currbyte, 1)) && currbyte < 17) {
		currbyte++;
	}

	if(0 > retval) {
		perror("Error reading from v1 serial port (2)");
		return 1;
	}


	if(currbyte != 17) {
		fprintf(stderr, "Didn't get right number of bytes from serialport");
		return 1;
	}

	int idx = 0;
	if(NULL != loud)
		*loud = '0'==buf[idx++]?0:1;

	if(NULL != strength)
		*strength = buf[idx++] - '0';

	if(NULL != ledX)
		*ledX = '0'==buf[idx++]?0:1;

	if(NULL != ledK)
		*ledK = '0'==buf[idx++]?0:1;

	if(NULL != ledKA)
		*ledKA = '0'==buf[idx++]?0:1;

	if(NULL != ledLaser)
		*ledLaser = '0'==buf[idx++]?0:1;

	int i;
	for(i=0;i<8;i++) {
		 eightsegment[i] = '0'==buf[idx++]?0:1;
	}

	if(NULL != frontarrow)
		*frontarrow = '0'==buf[idx++]?0:1;

	if(NULL != sidearrow)
		*sidearrow = '0'==buf[idx++]?0:1;

	if(NULL != reararrow)
		*reararrow = '0'==buf[idx++]?0:1;

	return 0;
}


void printhelp(const char *argv0) {
	printf("Usage: %s [args]\n"
		"   [-m|--mute]\n"
		"   [-d|--device=<" V1CS_DEFAULT_DEVICE ">]\n"
		"   [-1|--on]\n"
		"   [-0|--off]\n"
		"   [-l|--listen]\n"
		"   [-o|--mode]\n"
		"   [-v|--version]\n"
		"   [-h|--help]\n"
			, argv0);
}
void printversion() {
	printf("Version: 0\n");
}

